@@ -22,7 +22,7 @@ if confighelp then
   rspamd_config:add_example(nil, 'gpt',
       "Performs postfiltering using GPT model",
       [[
-gpt {
+  gpt {
   # Supported types: openai, ollama
   type = "openai";
   # Your key to access the API
@@ -53,7 +53,7 @@ gpt {
   reason_header = "X-GPT-Reason";
   # Use JSON format for response
   json = false;
-}
+  }
   ]])
   return
 end
@@ -355,14 +355,27 @@ local function default_openai_plain_conversion(task, input)
   local reason = clean_reply_line(lines[2])
   local categories = lua_util.str_split(clean_reply_line(lines[3]), ',')
 
+  if type(reply.usage) == 'table' then
+    rspamd_logger.infox(task, 'usage: %s tokens', reply.usage.total_tokens)
+  end
+
   if spam_score then
     return spam_score, reason, categories
   end
 
-  rspamd_logger.errx(task, 'cannot parse plain gpt reply: %s (all: %s)', lines[1])
+  rspamd_logger.errx(task, 'cannot parse plain gpt reply: %s (all: %s)', lines[1], first_message)
   return
 end
 
+-- Helper function to remove <think>...</think> and trim leading newlines
+local function clean_gpt_response(text)
+  -- Remove <think>...</think> including multiline
+  text = text:gsub("<think>.-</think>", "")
+  -- Trim leading whitespace and newlines
+  text = text:gsub("^%s*\n*", "")
+  return text
+end
+
 local function default_ollama_plain_conversion(task, input)
   local parser = ucl.parser()
   local res, err = parser:parse_string(input)
@@ -387,6 +400,10 @@ local function default_ollama_plain_conversion(task, input)
     rspamd_logger.errx(task, 'no content in the first message')
     return
   end
+
+  -- Clean message
+  first_message = clean_gpt_response(first_message)
+
   local lines = lua_util.str_split(first_message, '\n')
   local first_line = clean_reply_line(lines[1])
   local spam_score = tonumber(first_line)
@@ -397,7 +414,7 @@ local function default_ollama_plain_conversion(task, input)
     return spam_score, reason, categories
   end
 
-  rspamd_logger.errx(task, 'cannot parse plain gpt reply: %s', lines[1])
+  rspamd_logger.errx(task, 'cannot parse plain gpt reply: %s (all: %s)', lines[1], first_message)
   return
 end
 
@@ -494,6 +511,7 @@ local function insert_results(task, result, sel_part)
     rspamd_logger.errx(task, 'no probability in result')
     return
   end
+
   if result.probability > 0.5 then
     task:insert_result('GPT_SPAM', (result.probability - 0.5) * 2, tostring(result.probability))
     if settings.autolearn then
@@ -504,10 +522,6 @@ local function insert_results(task, result, sel_part)
       process_categories(task, result.categories)
     end
   else
-    if result.reason and settings.reason_header then
-      lua_mime.modify_headers(task,
-          { add = { [settings.reason_header] = { value = 'value', order = 1 } } })
-    end
     task:insert_result('GPT_HAM', (0.5 - result.probability) * 2, tostring(result.probability))
     if settings.autolearn then
       task:set_flag("learn_ham")
@@ -516,6 +530,10 @@ local function insert_results(task, result, sel_part)
       process_categories(task, result.categories)
     end
   end
+  if result.reason and settings.reason_header then
+    lua_mime.modify_headers(task,
+        { add = { [settings.reason_header] = { value = tostring(result.reason), order = 1 } } })
+  end
 
   if cache_context then
     lua_cache.cache_set(task, redis_cache_key(sel_part), result, cache_context)
@@ -807,6 +825,10 @@ local function ollama_check(task, content, sel_part)
       body.response_format = { type = "json_object" }
     end
 
+    results[i] = {
+      success = false,
+      checked = false
+    }
     body.model = model
 
     upstream = settings.upstreams:get_upstream_round_robin()
@@ -954,14 +976,14 @@ if opts then
           "FROM and url domains. Evaluate spam probability (0-1). " ..
           "Output ONLY 3 lines:\n" ..
           "1. Numeric score (0.00-1.00)\n" ..
-          "2. One-sentence reason citing strongest red flag\n" ..
+          "2. One-sentence reason citing whether it is spam, the strongest red flag, or why it is ham\n" ..
           "3. Primary concern category if found from the list: " .. table.concat(lua_util.keys(categories_map), ', ')
     else
       settings.prompt = "Analyze this email strictly as a spam detector given the email message, subject, " ..
           "FROM and url domains. Evaluate spam probability (0-1). " ..
           "Output ONLY 2 lines:\n" ..
           "1. Numeric score (0.00-1.00)\n" ..
-          "2. One-sentence reason citing strongest red flag\n"
+          "2. One-sentence reason citing whether it is spam, the strongest red flag, or why it is ham\n"
     end
   end
 end
