@@ -1267,6 +1267,7 @@ local function add_multimap_rule(key, newrule)
         {
           rules = newrule.rules,
           expression = newrule.expression,
+          description = newrule.description,
           on_load = newrule.dynamic_symbols and multimap_on_load_gen(newrule) or nil,
         }, N, 'Combined map for ' .. newrule.symbol)
     if not newrule.combined then
@@ -1332,6 +1333,9 @@ local function add_multimap_rule(key, newrule)
       end
     elseif newrule['type'] == 'dnsbl' then
       ret = true
+    else
+      rspamd_logger.errx(rspamd_config, 'cannot add rule %s: invalid type %s',
+          key, newrule['type'])
     end
   end
 
