@@ -40,6 +40,7 @@ local N = 'rbl'
 
 -- Checks that could be performed by rbl module
 local local_exclusions
+local disabled_rbl_suffixes -- Map of disabled rbl suffixes
 local white_symbols = {}
 local black_symbols = {}
 local monitored_addresses = {}
@@ -83,16 +84,20 @@ end
 local function validate_dns(lstr)
   if lstr:match('%.%.') then
     -- two dots in a row
-    return false
+    return false, "two dots in a row"
   end
   if not rspamd_util.is_valid_utf8(lstr) then
     -- invalid utf8 detected
-    return false
+    return false, "invalid utf8"
   end
   for v in lstr:gmatch('[^%.]+') do
-    if v:len() > 63 or v:match('^-') or v:match('-$') then
-      -- too long label or weird labels
-      return false
+    if v:len() > 63 then
+      -- too long label
+      return false, "too long label"
+    end
+    if v:match('^-') or v:match('-$') then
+      -- dash at the beginning or end of label
+      return false, "dash at the beginning or end of label"
     end
   end
   return true
@@ -142,12 +147,12 @@ local function gen_check_rcvd_conditions(rbl, received_total)
   local nmatch_flags = rbl.received_nflags
 
   local function basic_received_check(rh)
-    if not (rh.real_ip and rh.real_ip:is_valid()) then return false end
+    if not (rh.real_ip and rh.real_ip:is_valid()) then
+      return false
+    end
     if ((rh.real_ip:get_version() == 6 and rbl.ipv6) or
         (rh.real_ip:get_version() == 4 and rbl.ipv4)) and
-        ((rbl.exclude_private_ips and not rh.real_ip:is_local()) or
-            not rbl.exclude_private_ips) and ((rbl.exclude_local_ips and
-        not is_excluded_ip(rh.real_ip)) or not rbl.exclude_local_ips) then
+        ((rbl.exclude_local and not rh.real_ip:is_local() or is_excluded_ip(rh.real_ip)) or not rbl.exclude_local) then
       return true
     else
       return false
@@ -155,7 +160,9 @@ local function gen_check_rcvd_conditions(rbl, received_total)
   end
 
   local function positioned_received_check(rh, pos)
-    if not rh or not basic_received_check(rh) then return false end
+    if not rh or not basic_received_check(rh) then
+      return false
+    end
     local got_flags = rh.flags or E
     if min_pos then
       if min_pos < 0 then
@@ -200,7 +207,6 @@ local function gen_check_rcvd_conditions(rbl, received_total)
     return true
   end
 
-
   if not (max_pos or min_pos or match_flags or nmatch_flags) then
     return basic_received_check
   else
@@ -208,7 +214,45 @@ local function gen_check_rcvd_conditions(rbl, received_total)
   end
 end
 
-local function rbl_dns_process(task, rbl, to_resolve, results, err, resolve_table_elt)
+local matchers = {}
+
+matchers.radix = function(_, _, real_ip, map)
+  return map and map:get_key(real_ip) or false
+end
+
+matchers.equality = function(codes, to_match)
+  if type(codes) ~= 'table' then
+    return codes == to_match
+  end
+  for _, ip in ipairs(codes) do
+    if to_match == ip then
+      return true
+    end
+  end
+  return false
+end
+
+matchers.luapattern = function(codes, to_match)
+  if type(codes) ~= 'table' then
+    return string.find(to_match, '^' .. codes .. '$') and true or false
+  end
+  for _, pattern in ipairs(codes) do
+    if string.find(to_match, '^' .. pattern .. '$') then
+      return true
+    end
+  end
+  return false
+end
+
+matchers.regexp = function(_, to_match, _, map)
+  return map and map:get_key(to_match) or false
+end
+
+matchers.glob = function(_, to_match, _, map)
+  return map and map:get_key(to_match) or false
+end
+
+local function rbl_dns_process(task, rbl, to_resolve, results, err, resolve_table_elt, match)
   local function make_option(ip, label)
     if ip then
       return string.format('%s:%s:%s',
@@ -267,15 +311,17 @@ local function rbl_dns_process(task, rbl, to_resolve, results, err, resolve_tabl
     return
   end
 
-  for _,result in ipairs(results) do
+  local returncodes_maps = rbl.returncodes_maps or {}
+
+  for _, result in ipairs(results) do
     local ipstr = result:to_string()
     lua_util.debugm(N, task, '%s DNS result %s', to_resolve, ipstr)
     local foundrc = false
     -- Check return codes
     if rbl.returnbits then
       local ipnum = result:to_number()
-      for s,bits in pairs(rbl.returnbits) do
-        for _,check_bit in ipairs(bits) do
+      for s, bits in pairs(rbl.returnbits) do
+        for _, check_bit in ipairs(bits) do
           if bit.band(ipnum, check_bit) == check_bit then
             foundrc = true
             insert_results(s)
@@ -285,12 +331,10 @@ local function rbl_dns_process(task, rbl, to_resolve, results, err, resolve_tabl
       end
     elseif rbl.returncodes then
       for s, codes in pairs(rbl.returncodes) do
-        for _,v in ipairs(codes) do
-          if string.find(ipstr, '^' .. v .. '$') then
-            foundrc = true
-            insert_results(s)
-            break
-          end
+        local res = match(codes, ipstr, result, returncodes_maps[s])
+        if res then
+          foundrc = true
+          insert_results(s)
         end
       end
     end
@@ -370,7 +414,7 @@ local function gen_rbl_callback(rule)
         nreq.what[label] = true
       end
 
-      return true,nreq -- Duplicate
+      return true, nreq -- Duplicate
     else
       local nreq
 
@@ -384,7 +428,7 @@ local function gen_rbl_callback(rule)
             n = processed,
             orig = req_str,
             resolve_ip = resolve_ip,
-            what = {[label] = true},
+            what = { [label] = true },
           }
           requests_table[req] = nreq
         end
@@ -407,7 +451,7 @@ local function gen_rbl_callback(rule)
           n = to_resolve,
           orig = req_str,
           resolve_ip = resolve_ip,
-          what = {[label] = true},
+          what = { [label] = true },
         }
         requests_table[req] = nreq
       end
@@ -429,9 +473,22 @@ local function gen_rbl_callback(rule)
     return true
   end
 
+  local function is_allowed(task, _)
+    if disabled_rbl_suffixes then
+      if disabled_rbl_suffixes:get_key(rule.rbl) then
+        lua_util.debugm(N, task, 'skip rbl check: %s; disabled by suffix', rule.rbl)
+        return false
+      end
+    end
+
+    return true
+  end
+
   local function check_required_symbols(task, _)
     if rule.require_symbols then
-      return fun.all(function(sym) task:has_symbol(sym) end, rule.require_symbols)
+      return fun.all(function(sym)
+        task:has_symbol(sym)
+      end, rule.require_symbols)
     end
 
     return true
@@ -484,7 +541,7 @@ local function gen_rbl_callback(rule)
         mime_from_domain = ((task:get_from('mime') or E)[1] or E).domain
         if mime_from_domain then
           local mime_from_domain_tld = rule.url_full_hostname and
-               mime_from_domain or rspamd_util.get_tld(mime_from_domain)
+              mime_from_domain or rspamd_util.get_tld(mime_from_domain)
 
           if rule.url_compose_map then
             mime_from_domain = rule.url_compose_map:process_url(task, mime_from_domain_tld, mime_from_domain)
@@ -496,7 +553,7 @@ local function gen_rbl_callback(rule)
 
       for _, d in ipairs(das[1].options) do
 
-        local domain,result = d:match('^([^%:]*):([%+%-%~])$')
+        local domain, result = d:match('^([^%:]*):([%+%-%~])$')
 
         -- We must ignore bad signatures, omg
         if domain and result and result == '+' then
@@ -516,7 +573,7 @@ local function gen_rbl_callback(rule)
 
             if mime_from_domain and mime_from_domain == domain_tld then
               add_dns_request(task, domain_tld, true, false, requests_table,
-              'dkim', whitelist)
+                  'dkim', whitelist)
             end
           else
             if rule.dkim_domainonly then
@@ -553,12 +610,36 @@ local function gen_rbl_callback(rule)
       ignore_ip = rule.no_ip,
       need_images = rule.images,
       need_emails = false,
-      need_content = rule.content_urls or false,
+      need_content = rule.content_urls,
       esld_limit = esld_lim,
       no_cache = true,
     }
 
-    if not rule.urls then
+    if rule.numeric_urls then
+      if rule.content_urls then
+        if not rule.images then
+          ex_params.flags_mode = 'explicit'
+          ex_params.flags = { 'numeric' }
+          ex_params.filter = function(url)
+            return (bit.band(url:get_flags_num(), url_flag_bits.image) == 0)
+          end
+        else
+          ex_params.filter = function(url)
+            return (bit.band(url:get_flags_num(), url_flag_bits.numeric) ~= 0)
+          end
+        end
+      elseif rule.images then
+        ex_params.filter = function(url)
+          return (bit.band(url:get_flags_num(), url_flag_bits.numeric) ~= 0)
+        end
+      else
+        ex_params.flags_mode = 'explicit'
+        ex_params.flags = { 'numeric' }
+        ex_params.filter = function(url)
+          return (bit.band(url:get_flags_num(), url_flag_bits.content) == 0)
+        end
+      end
+    elseif not rule.urls and (rule.content_urls or rule.images) then
       ex_params.flags_mode = 'explicit'
       ex_params.flags = {}
       if rule.content_urls then
@@ -571,7 +652,7 @@ local function gen_rbl_callback(rule)
 
     local urls = lua_util.extract_specific_urls(ex_params)
 
-    for _,u in ipairs(urls) do
+    for _, u in ipairs(urls) do
       local flags = u:get_flags_num()
 
       if bit.band(flags, url_flag_bits.numeric) ~= 0 then
@@ -616,18 +697,25 @@ local function gen_rbl_callback(rule)
   end
 
   local function check_received(task, requests_table, whitelist)
-    local received = fun.filter(function(h)
+    local received = fun             .filter(function(h)
       return not h['flags']['artificial']
     end, task:get_received_headers()):totable()
 
     local received_total = #received
     local check_conditions = gen_check_rcvd_conditions(rule, received_total)
+    local from_ip = task:get_from_ip()
 
-    for pos,rh in ipairs(received) do
+    for pos, rh in ipairs(received) do
       if check_conditions(rh, pos) then
-        add_dns_request(task, rh.real_ip, false, true,
-            requests_table, 'received',
-            whitelist)
+        if rh.real_ip ~= from_ip or (rh.real_ip == from_ip and not rule.from) then
+          add_dns_request(task, rh.real_ip, false, true,
+              requests_table, 'received',
+              whitelist)
+        else
+          lua_util.debugm(N, task, 'rbl %s; skip check_received for %s:' ..
+              'Received IP same as From IP and will be checked only in check_from function',
+              rule.symbol, rh.real_ip)
+        end
       end
     end
 
@@ -651,13 +739,13 @@ local function gen_rbl_callback(rule)
       local res = selector(task)
 
       if res and type(res) == 'table' then
-          for _,r in ipairs(res) do
-            add_dns_request(task, r, false, false, requests_table,
-                    selector_label, whitelist)
-          end
+        for _, r in ipairs(res) do
+          add_dns_request(task, r, false, false, requests_table,
+              selector_label, whitelist)
+        end
       elseif res then
         add_dns_request(task, res, false, false,
-                requests_table, selector_label, whitelist)
+            requests_table, selector_label, whitelist)
       end
     end
 
@@ -705,7 +793,9 @@ local function gen_rbl_callback(rule)
     local ex_params = {
       task = task,
       limit = rule.requests_limit,
-      filter = function(u) return u:get_protocol() == 'mailto' end,
+      filter = function(u)
+        return u:get_protocol() == 'mailto'
+      end,
       need_emails = true,
       prefix = 'rbl_email'
     }
@@ -719,7 +809,7 @@ local function gen_rbl_callback(rule)
 
     local emails = lua_util.extract_specific_urls(ex_params)
 
-    for _,email in ipairs(emails) do
+    for _, email in ipairs(emails) do
       local domain
       if rule.emails_domainonly and not rule.url_full_hostname then
         if rule.url_compose_map then
@@ -762,6 +852,7 @@ local function gen_rbl_callback(rule)
 
   -- Create function pipeline depending on rbl settings
   local pipeline = {
+    is_allowed, -- check if rbl is allowed
     is_alive, -- check monitored status
     check_required_symbols -- if we have require_symbols then check those symbols
   }
@@ -774,7 +865,7 @@ local function gen_rbl_callback(rule)
     description[#description + 1] = 'user'
   end
 
-  if rule.exclude_local or rule.exclude_private_ips then
+  if rule.exclude_local then
     pipeline[#pipeline + 1] = check_local
     description[#description + 1] = 'local'
   end
@@ -798,7 +889,7 @@ local function gen_rbl_callback(rule)
     description[#description + 1] = 'replyto'
   end
 
-  if rule.urls or rule.content_urls or rule.images then
+  if rule.urls or rule.content_urls or rule.images or rule.numeric_urls then
     pipeline[#pipeline + 1] = check_urls
     description[#description + 1] = 'urls'
   end
@@ -823,6 +914,11 @@ local function gen_rbl_callback(rule)
     description[#description + 1] = 'selector'
   end
 
+  if not rule.returncodes_matcher then
+    rule.returncodes_matcher = 'equality'
+  end
+  local match = matchers[rule.returncodes_matcher]
+
   local callback_f = function(task)
     -- DNS requests to issue (might be hashed afterwards)
     local dns_req = {}
@@ -830,12 +926,12 @@ local function gen_rbl_callback(rule)
 
     local function gen_rbl_dns_callback(resolve_table_elt)
       return function(_, to_resolve, results, err)
-        rbl_dns_process(task, rule, to_resolve, results, err, resolve_table_elt)
+        rbl_dns_process(task, rule, to_resolve, results, err, resolve_table_elt, match)
       end
     end
 
     -- Execute functions pipeline
-    for i,f in ipairs(pipeline) do
+    for i, f in ipairs(pipeline) do
       if not f(task, dns_req, whitelist) then
         lua_util.debugm(N, task,
             "skip rbl check: %s; pipeline condition %s returned false",
@@ -854,12 +950,12 @@ local function gen_rbl_callback(rule)
     local function gen_rbl_ip_dns_callback(orig_resolve_table_elt)
       return function(_, _, results, err)
         if not err then
-          for _,dns_res in ipairs(results) do
+          for _, dns_res in ipairs(results) do
             -- Check if we have rspamd{ip} userdata
             if type(dns_res) == 'userdata' then
               -- Add result as an actual RBL request
               local label = next(orig_resolve_table_elt.what)
-              local dup,nreq = add_dns_request(task, dns_res, false, true,
+              local dup, nreq = add_dns_request(task, dns_res, false, true,
                   resolved_req, label)
               -- Add original name
               if not dup then
@@ -874,7 +970,8 @@ local function gen_rbl_callback(rule)
         if nresolved == 0 then
           -- Emit real RBL requests as there are no ip resolution requests
           for name, req in pairs(resolved_req) do
-            if validate_dns(req.n) then
+            local val_res, val_error = validate_dns(req.n)
+            if val_res then
               lua_util.debugm(N, task, "rbl %s; resolve %s -> %s",
                   rule.symbol, name, req.n)
               r:resolve_a({
@@ -884,8 +981,8 @@ local function gen_rbl_callback(rule)
                 forced = req.forced
               })
             else
-              rspamd_logger.warnx(task, 'cannot send invalid DNS request %s for %s',
-                  req.n, rule.symbol)
+              rspamd_logger.warnx(task, 'cannot send invalid DNS request %s for %s: %s',
+                  req.n, rule.symbol, val_error)
             end
           end
         end
@@ -893,14 +990,15 @@ local function gen_rbl_callback(rule)
     end
 
     for name, req in pairs(dns_req) do
-      if validate_dns(req.n) then
+      local val_res, val_error = validate_dns(req.n)
+      if val_res then
         lua_util.debugm(N, task, "rbl %s; resolve %s -> %s",
             rule.symbol, name, req.n)
 
         if req.resolve_ip then
           -- Deal with both ipv4 and ipv6
           -- Resolve names first
-          if r:resolve_a({
+          if (rule.ipv4 == nil or rule.ipv4) and r:resolve_a({
             task = task,
             name = req.n,
             callback = gen_rbl_ip_dns_callback(req),
@@ -908,7 +1006,7 @@ local function gen_rbl_callback(rule)
           }) then
             nresolved = nresolved + 1
           end
-          if r:resolve('aaaa', {
+          if (rule.ipv6 == nil or rule.ipv6) and r:resolve('aaaa', {
             task = task,
             name = req.n,
             callback = gen_rbl_ip_dns_callback(req),
@@ -926,21 +1024,27 @@ local function gen_rbl_callback(rule)
         end
 
       else
-        rspamd_logger.warnx(task, 'cannot send invalid DNS request %s for %s',
-            req.n, rule.symbol)
+        rspamd_logger.warnx(task, 'cannot send invalid DNS request %s for %s: %s',
+            req.n, rule.symbol, val_error)
       end
     end
   end
 
-  return callback_f,string.format('checks: %s', table.concat(description, ','))
+  return callback_f, string.format('checks: %s', table.concat(description, ','))
 end
 
+local map_match_types = {
+  glob = true,
+  radix = true,
+  regexp = true,
+}
+
 local function add_rbl(key, rbl, global_opts)
   if not rbl.symbol then
     rbl.symbol = key:upper()
   end
 
-  local flags_tbl = {'no_squeeze'}
+  local flags_tbl = { 'no_squeeze' }
   if rbl.is_whitelist then
     flags_tbl[#flags_tbl + 1] = 'nice'
   end
@@ -955,7 +1059,7 @@ local function add_rbl(key, rbl, global_opts)
 
     rbl.selectors = {}
     if type(rbl.selector) ~= 'table' then
-      rbl.selector = {['selector'] = rbl.selector}
+      rbl.selector = { ['selector'] = rbl.selector }
     end
 
     for selector_label, selector in pairs(rbl.selector) do
@@ -970,7 +1074,7 @@ local function add_rbl(key, rbl, global_opts)
           rbl.selector_flatten = true
         end
         local sel = selectors.create_selector_closure(rspamd_config, selector, '',
-                rbl.selector_flatten)
+            rbl.selector_flatten)
 
         if not sel then
           rspamd_logger.errx('invalid selector for rbl rule %s: %s', key, selector)
@@ -1012,6 +1116,20 @@ local function add_rbl(key, rbl, global_opts)
         def_type, rbl.symbol)
   end
 
+  local match_type = rbl.returncodes_matcher
+  if match_type and rbl.returncodes and map_match_types[match_type] then
+    if not rbl.returncodes_maps then
+      rbl.returncodes_maps = {}
+    end
+    for label, v in pairs(rbl.returncodes) do
+      if type(v) ~= 'table' then
+        v = { v }
+      end
+      rbl.returncodes_maps[label] = lua_maps.map_add_from_ucl(v, match_type,
+          string.format('%s_%s RBL returncodes', label, rbl.symbol))
+    end
+  end
+
   if rbl.url_compose_map then
     local lua_urls_compose = require "lua_urls_compose"
     rbl.url_compose_map = lua_urls_compose.add_composition_map(rspamd_config, rbl.url_compose_map)
@@ -1022,33 +1140,33 @@ local function add_rbl(key, rbl, global_opts)
     end
   end
 
-  if not rbl.whitelist and global_opts.url_whitelist and
+  if not rbl.whitelist and not rbl.ignore_url_whitelist and (global_opts.url_whitelist or rbl.url_whitelist) and
       (rbl.urls or rbl.emails or rbl.dkim or rbl.replyto) and
       not (rbl.from or rbl.received) then
     local def_type = 'set'
-    rbl.whitelist = lua_maps.map_add_from_ucl(global_opts.url_whitelist, def_type,
+    rbl.whitelist = lua_maps.map_add_from_ucl(rbl.url_whitelist or global_opts.url_whitelist, def_type,
         'RBL url whitelist for ' .. rbl.symbol)
     rspamd_logger.infox(rspamd_config, 'added URL whitelist for RBL %s',
         rbl.symbol)
   end
 
-  local callback,description = gen_rbl_callback(rbl)
+  local callback, description = gen_rbl_callback(rbl)
 
   if callback then
     local id
 
     if rbl.symbols_prefixes then
-      id = rspamd_config:register_symbol{
+      id = rspamd_config:register_symbol {
         type = 'callback',
         callback = callback,
-        groups = {'rbl'},
+        groups = { 'rbl' },
         name = rbl.symbol .. '_CHECK',
         flags = table.concat(flags_tbl, ',')
       }
 
-      for _,prefix in pairs(rbl.symbols_prefixes) do
+      for _, prefix in pairs(rbl.symbols_prefixes) do
         -- For unknown results...
-        rspamd_config:register_symbol{
+        rspamd_config:register_symbol {
           type = 'virtual',
           parent = id,
           group = 'rbl',
@@ -1064,11 +1182,11 @@ local function add_rbl(key, rbl, global_opts)
             rbl.symbol, rbl.is_whitelist, rbl.ignore_whitelist)
       end
     else
-      id = rspamd_config:register_symbol{
+      id = rspamd_config:register_symbol {
         type = 'callback',
         callback = callback,
         name = rbl.symbol,
-        groups = {'rbl'},
+        groups = { 'rbl' },
         group = 'rbl',
         score = 0,
         flags = table.concat(flags_tbl, ',')
@@ -1082,26 +1200,25 @@ local function add_rbl(key, rbl, global_opts)
       end
     end
 
-
     rspamd_logger.infox(rspamd_config, 'added rbl rule %s: %s',
         rbl.symbol, description)
     lua_util.debugm(N, rspamd_config, 'rule dump for %s: %s',
         rbl.symbol, rbl)
 
-    local check_sym = rbl.symbol_prefixes and rbl.symbol .. '_CHECK' or rbl.symbol
+    local check_sym = rbl.symbols_prefixes and rbl.symbol .. '_CHECK' or rbl.symbol
 
     if rbl.dkim then
       rspamd_config:register_dependency(check_sym, 'DKIM_CHECK')
     end
 
     if rbl.require_symbols then
-      for _,dep in ipairs(rbl.require_symbols) do
+      for _, dep in ipairs(rbl.require_symbols) do
         rspamd_config:register_dependency(check_sym, dep)
       end
     end
 
     -- Failure symbol
-    rspamd_config:register_symbol{
+    rspamd_config:register_symbol {
       type = 'virtual',
       flags = 'nostat',
       name = rbl.symbol .. '_FAIL',
@@ -1114,7 +1231,7 @@ local function add_rbl(key, rbl, global_opts)
         if s ~= rbl.symbol then
           -- hack
 
-          rspamd_config:register_symbol{
+          rspamd_config:register_symbol {
             type = 'virtual',
             parent = id,
             name = s,
@@ -1145,7 +1262,7 @@ local function add_rbl(key, rbl, global_opts)
       end
 
       if rbl.symbols_prefixes then
-        for _,prefix in pairs(rbl.symbols_prefixes) do
+        for _, prefix in pairs(rbl.symbols_prefixes) do
           process_specific_suffix(prefix .. '_' .. suffix)
         end
       else
@@ -1155,13 +1272,13 @@ local function add_rbl(key, rbl, global_opts)
     end
 
     if rbl.returncodes then
-      for s,_ in pairs(rbl.returncodes) do
+      for s, _ in pairs(rbl.returncodes) do
         process_return_code(s)
       end
     end
 
     if rbl.returnbits then
-      for s,_ in pairs(rbl.returnbits) do
+      for s, _ in pairs(rbl.returnbits) do
         process_return_code(s)
       end
     end
@@ -1200,16 +1317,16 @@ if opts.rules and opts.rbls then
   opts.rbls = lua_util.override_defaults(opts.rbls, opts.rules)
 end
 
-if(opts['local_exclude_ip_map'] ~= nil) then
+if (opts['local_exclude_ip_map'] ~= nil) then
   local_exclusions = lua_maps.map_add(N, 'local_exclude_ip_map', 'radix',
-    'RBL exclusions map')
+      'RBL exclusions map')
 end
 
 -- TODO: this code should be universal for all modules that use selectors to allow
 -- maps usage from selectors registered for a specific module
 if type(opts.attached_maps) == 'table' then
   opts.attached_maps_processed = {}
-  for i,map in ipairs(opts.attached_maps) do
+  for i, map in ipairs(opts.attached_maps) do
     -- Store maps in the configuration table to keep lifetime track
     opts.attached_maps_processed[i] = lua_maps.map_add_from_ucl(map)
     if opts.attached_maps_processed[i] == nil then
@@ -1218,7 +1335,12 @@ if type(opts.attached_maps) == 'table' then
   end
 end
 
-for key,rbl in pairs(opts.rbls) do
+if opts.disabled_rbl_suffixes_map then
+  disabled_rbl_suffixes = lua_maps.map_add_from_ucl(opts.disabled_rbl_suffixes_map, 'set',
+      'Disabled suffixes for RBL')
+end
+
+for key, rbl in pairs(opts.rbls) do
   if type(rbl) ~= 'table' or rbl.disabled == true or rbl.enabled == false then
     rspamd_logger.infox(rspamd_config, 'disable rbl "%s"', key)
   else
@@ -1231,7 +1353,7 @@ for key,rbl in pairs(opts.rbls) do
     end
     -- Propagate default options from opts to rule
     if not rbl.ignore_defaults then
-      for default_opt_key,_ in pairs(rbl_common.default_options) do
+      for default_opt_key, _ in pairs(rbl_common.default_options) do
         local rbl_opt = default_opt_key:sub(#('default_') + 1)
         if rbl[rbl_opt] == nil then
           rbl[rbl_opt] = opts[default_opt_key]
@@ -1243,15 +1365,19 @@ for key,rbl in pairs(opts.rbls) do
       rbl.requests_limit = rspamd_config:get_dns_max_requests()
     end
 
-    local res,err = rbl_common.rule_schema:transform(rbl)
+    local res, err = rbl_common.rule_schema:transform(rbl)
     if not res then
       rspamd_logger.errx(rspamd_config, 'invalid config for %s: %s, RBL is DISABLED',
           key, err)
     else
-      res = rbl_common.convert_checks(res)
+      res = rbl_common.convert_checks(res, rbl.symbol or key:upper())
       -- Aliases
-      if res.return_codes then res.returncodes = res.return_codes end
-      if res.return_bits then res.returnbits = res.return_bits end
+      if res.return_codes then
+        res.returncodes = res.return_codes
+      end
+      if res.return_bits then
+        res.returnbits = res.return_bits
+      end
 
       if not res then
         rspamd_logger.errx(rspamd_config, 'invalid config for %s: %s, RBL is DISABLED',
@@ -1272,10 +1398,12 @@ local function rbl_callback_white(task)
     local ws = task:get_symbol(w)
     if ws and ws[1] then
       ws = ws[1]
-      if not ws.options then ws.options = {} end
-      for _,opt in ipairs(ws.options) do
-        local elt,what = opt:match('^([^:]+):([^:]+)')
-        lua_util.debugm(N, task,'found whitelist from %s: %s(%s)', w,
+      if not ws.options then
+        ws.options = {}
+      end
+      for _, opt in ipairs(ws.options) do
+        local elt, what = opt:match('^([^:]+):([^:]+)')
+        lua_util.debugm(N, task, 'found whitelist from %s: %s(%s)', w,
             elt, what)
         if elt and what then
           whitelisted_elements[elt] = {
@@ -1297,22 +1425,22 @@ local function rbl_callback_fin(task)
   lua_util.debugm(N, task, "finished rbl processing")
 end
 
-rspamd_config:register_symbol{
+rspamd_config:register_symbol {
   type = 'callback',
   callback = rbl_callback_white,
   name = 'RBL_CALLBACK_WHITE',
   flags = 'nice,empty,no_squeeze',
-  groups = {'rbl'},
-  augmentations = {string.format("timeout=%f", rspamd_config:get_dns_timeout() or 0.0)},
+  groups = { 'rbl' },
+  augmentations = { string.format("timeout=%f", rspamd_config:get_dns_timeout() or 0.0) },
 }
 
-rspamd_config:register_symbol{
+rspamd_config:register_symbol {
   type = 'callback',
   callback = rbl_callback_fin,
   name = 'RBL_CALLBACK',
   flags = 'empty,no_squeeze',
-  groups = {'rbl'},
-  augmentations = {string.format("timeout=%f", rspamd_config:get_dns_timeout() or 0.0)},
+  groups = { 'rbl' },
+  augmentations = { string.format("timeout=%f", rspamd_config:get_dns_timeout() or 0.0) },
 }
 
 for _, w in ipairs(white_symbols) do
